// Copyright 2013 the V8 project authors. All rights reserved.
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above
//       copyright notice, this list of conditions and the following
//       disclaimer in the documentation and/or other materials provided
//       with the distribution.
//     * Neither the name of Google Inc. nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// Flags: --allow-natives-syntax

"use strict";

// Function is defined on the prototype chain.
var holder = {
  f: function() {
    return 42;
  }
};
var receiver = {};
receiver.__proto__ = {};
receiver.__proto__.__proto__ = holder;

// Inline two levels.
function h(o) {
  return o.f.apply(this, arguments);
}
function g(o) {
  return h(o);
}

// Collect type information for apply call.
;
%PrepareFunctionForOptimization(g);
assertEquals(42, g(receiver));
assertEquals(42, g(receiver));

// Sneakily remove the function from the prototype chain.
// The receiver map does not change.
receiver.__proto__.__proto__ = {};

// Lookup of o.f during graph creation fails.
%OptimizeFunctionOnNextCall(g);

assertThrows(function() {
  g(receiver);
});

// Put function back.
receiver.__proto__.__proto__ = holder;
assertEquals(42, g(receiver));
