"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * A patch for `fs` permitting certain directories to be routed within the
 * binary.
 */
var fs = require("fs");
var path = require("path");
var util = require("util");
var error_1 = require("../common/error");
/**
 * Fills `${pathName}/*` with the binary stuff.
 */
exports.fillFs = function (pathName, filesystem) {
    var openFiles = new Map();
    var override = function (propertyName, factory, customPromisify) {
        var oldfunc = fs[propertyName];
        fs[propertyName] = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var callOld = function () { return oldfunc.apply(void 0, args); };
            // Try to determine whether this call falls within the path we are trying
            // to fill. If it doesn't we'll call the original method.
            var realPath = args[0];
            // Likely a file descriptor
            if (typeof realPath === "number") {
                var newFd = args[0];
                var maybeRealPath = openFiles.get(newFd);
                if (maybeRealPath) {
                    realPath = maybeRealPath;
                }
                // Likely a file path.
            }
            else if (typeof realPath === "string") {
                var newPath = realPath;
                var rel = path.relative(pathName, newPath);
                if (!rel.startsWith("..")) {
                    // Looks like it falls within the fill path.
                    return factory(callOld).apply(void 0, args);
                }
            }
            return callOld();
        };
        if (customPromisify) {
            ;
            fs[propertyName][util.promisify.custom] = customPromisify;
        }
    };
    var fdId = 0;
    var findCb = function (args) {
        var cb = args.filter(function (d) {
            return typeof d === "function";
        });
        if (cb.length === 0) {
            return;
        }
        return cb[0];
    };
    override("access", function (callOld) { return function (pathName) {
        if (!filesystem.existsSync(pathName)) {
            return callOld();
        }
    }; });
    override("accessSync", function (callOld) { return function (pathName) {
        if (!filesystem.existsSync(pathName)) {
            return callOld();
        }
    }; });
    override("close", function (callOld) { return function (fd, callback) {
        if (!openFiles.has(fd)) {
            return callOld();
        }
        openFiles.delete(fd);
        callback(null);
    }; });
    override("closeSync", function (callOld) { return function (fd) {
        if (!openFiles.has(fd)) {
            return callOld();
        }
        openFiles.delete(fd);
    }; });
    override("exists", function () { return function (pathName, callback) {
        callback(filesystem.existsSync(pathName));
    }; }, function (pathName) {
        return new Promise(function (resolve) {
            return fs.exists(pathName, function (exists) {
                resolve(exists);
            });
        });
    });
    override("existsSync", function () { return function (pathName) {
        return filesystem.existsSync(pathName);
    }; });
    override("fstat", function (callOld) { return function (fd, callback) {
        if (!openFiles.has(fd)) {
            return callOld();
        }
        var openFile = openFiles.get(fd);
        if (!openFile) {
            return callback(error_1.createNotFound(), null);
        }
        return fs.stat(openFile.path, callback);
    }; });
    override("fstatSync", function (callOld) { return function (fd) {
        if (!openFiles.has(fd)) {
            return callOld();
        }
        var openFile = openFiles.get(fd);
        if (!openFile) {
            throw error_1.createNotFound();
        }
        return fs.statSync(openFile.path);
    }; });
    override("lstat", function () { return function (pathName, callback) {
        return fs.stat(pathName, callback);
    }; });
    override("lstatSync", function () { return function (pathName) {
        return fs.statSync(pathName);
    }; });
    var doOpen = function (pathName) {
        var desc = fdId++;
        openFiles.set(desc, {
            path: pathName,
            readLocation: 0,
        });
        return desc;
    };
    override("open", function (callOld) { return function (pathName) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (!filesystem.existsSync(pathName)) {
            return callOld();
        }
        var fd = doOpen(pathName);
        var cb = findCb(args);
        if (!cb) {
            return;
        }
        process.nextTick(function () {
            cb(null, fd);
        });
    }; });
    override("openSync", function (callOld) { return function (pathName) {
        if (!filesystem.existsSync(pathName)) {
            return callOld();
        }
        return doOpen(pathName);
    }; });
    override("read", function (callOld) { return function (fd, buffer, offset, length, position, callback) {
        var openFile = openFiles.get(fd);
        if (!openFile) {
            return callOld();
        }
        var hadPosition = true;
        if (typeof position === "undefined" || position === null) {
            position = openFile.readLocation;
            hadPosition = false;
        }
        filesystem
            .readFile(openFile.path, "buffer", position, length)
            .then(function (content) {
            buffer.set(content, offset);
            if (!hadPosition) {
                openFile.readLocation += content.byteLength;
            }
            // tslint:disable-next-line:no-any
            callback(null, content.byteLength, content);
        })
            .catch(function (ex) {
            callback(ex, null, null);
        });
    }; }, function (fd, buffer, offset, length, position) {
        return new Promise(function (resolve, reject) {
            return fs.read(fd, buffer, offset, length, position, function (err, bytesRead, buffer) {
                if (err) {
                    return reject(err);
                }
                resolve({
                    bytesRead: bytesRead,
                    buffer: buffer,
                });
            });
        });
    });
    override("readSync", function (callOld) { return function (fd, buffer, offset, length, position) {
        var openFile = openFiles.get(fd);
        if (!openFile) {
            return callOld();
        }
        var hadPosition = true;
        if (typeof position === "undefined" || position === null) {
            position = openFile.readLocation;
            hadPosition = false;
        }
        var content = filesystem.readFileSync(openFile.path, "buffer", position, length);
        buffer.set(content, offset);
        if (!hadPosition) {
            openFile.readLocation += content.byteLength;
        }
        return content.byteLength;
    }; });
    override("readdir", function () { return function (pathName) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var cb = findCb(args);
        if (!cb) {
            return;
        }
        cb(null, filesystem.readdirSync(pathName));
    }; });
    override("readdirSync", function () { return function (pathName) {
        return __spreadArrays(filesystem.readdirSync(pathName));
    }; });
    override("readFile", function () { return function (pathName) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var encoding = "buffer";
        if (typeof args[0] === "string") {
            encoding = args[0];
        }
        if (typeof args[0] === "object" && args[0] !== null) {
            var opts = args[0];
            if (opts.encoding) {
                encoding = opts.encoding;
            }
        }
        var cb = findCb(args);
        if (!cb) {
            return;
        }
        filesystem
            .readFile(pathName, encoding)
            .then(function (result) {
            cb(null, result);
        })
            .catch(function (ex) {
            cb(ex);
        });
    }; });
    override("readFileSync", function () { return function (pathName) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var encoding = "buffer";
        if (typeof args[0] === "string") {
            encoding = args[0];
        }
        if (typeof args[0] === "object" && args[0] !== null) {
            var opts = args[0];
            if (opts.encoding) {
                encoding = opts.encoding;
            }
        }
        return filesystem.readFileSync(pathName, encoding);
    }; });
    override("realpath", function () { return function (pathName) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var cb = findCb(args);
        if (!cb) {
            return;
        }
        cb(null, pathName);
    }; });
    override("realpathSync", function () { return function (pathName) {
        return pathName;
    }; });
    var doStat = function (pathName) {
        var stat = filesystem.statSync(pathName);
        var date = new Date();
        return new (/** @class */ (function () {
            function class_1() {
                this.atime = date;
                this.atimeMs = date.getTime();
                this.birthtime = date;
                this.birthtimeMs = date.getTime();
                this.blksize = null;
                this.blocks = null;
                this.ctime = date;
                this.ctimeMs = date.getTime();
                this.dev = null;
                this.gid = 0;
                this.ino = 0;
                this.mode = null;
                this.mtime = date;
                this.mtimeMs = date.getTime();
                this.nlink = null;
                this.rdev = null;
                this.size = stat.size;
                this.uid = 0;
            }
            class_1.prototype.isBlockDevice = function () {
                return false;
            };
            class_1.prototype.isCharacterDevice = function () {
                return false;
            };
            class_1.prototype.isDirectory = function () {
                return stat.isDirectory;
            };
            class_1.prototype.isFIFO = function () {
                return false;
            };
            class_1.prototype.isFile = function () {
                return stat.isFile;
            };
            class_1.prototype.isSocket = function () {
                return false;
            };
            class_1.prototype.isSymbolicLink = function () {
                return false;
            };
            return class_1;
        }()))();
    };
    override("stat", function () { return function (pathName) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var cb = findCb(args);
        if (!cb) {
            return;
        }
        cb(null, doStat(pathName));
    }; });
    override("statSync", function () { return function (pathName) {
        return doStat(pathName);
    }; });
};
//# sourceMappingURL=fs.js.map