"use strict";
var fs = require("fs");
var path = require("path");
var buffer_1 = require("../common/buffer");
var filesystem_1 = require("../common/filesystem");
var footer_1 = require("../common/footer");
var util_1 = require("../common/util");
var fs_1 = require("./fs");
var execPath = process.execPath;
var execPathStat = fs.statSync(execPath);
var nbinFd = fs.openSync(execPath, "r");
// Footer is located at the end of the file.
var footer = footer_1.readFooter(nbinFd, execPathStat.size);
// Contains the version, mainFile and the filesystem.
var headerBuffer = Buffer.allocUnsafe(footer.headerLength);
fs.readSync(nbinFd, headerBuffer, 0, footer.headerLength, footer.headerOffset);
var version = buffer_1.readString(headerBuffer, 0);
var mainFile = buffer_1.readString(headerBuffer, version.offset);
// Maximize read performance by using the originals before any overrides.
var originalRead = fs.read;
var originalReadSync = fs.readSync;
var fsBuffer = headerBuffer.slice(mainFile.offset);
var readableFs = filesystem_1.ReadableFilesystem.fromBuffer(fsBuffer, {
    readContents: function (offset, length) {
        var buffer = Buffer.allocUnsafe(length);
        return new Promise(function (resolve, reject) {
            originalRead(nbinFd, buffer, 0, length, offset + footer.contentOffset, function (err, _, buffer) {
                if (err) {
                    return reject(err);
                }
                resolve(buffer);
            });
        });
    },
    readContentsSync: function (offset, length) {
        var buffer = Buffer.alloc(length);
        originalReadSync(nbinFd, buffer, 0, length, offset + footer.contentOffset);
        return buffer;
    },
});
var tmpDir = path.join(util_1.getXdgCacheHome("nbin"), version.value);
util_1.mkdirp(tmpDir);
var exported = {
    version: version.value,
    mainFile: mainFile.value,
    fs: readableFs,
    shimNativeFs: function (path) {
        fs_1.fillFs(path, readableFs);
    },
    tmpDir: tmpDir,
};
module.exports = exported;
//# sourceMappingURL=nbin.js.map